using System;
using System.Collections.Generic;
using System.Text;
using System.Runtime.InteropServices;
using SiteKioskRuntimeLib;

namespace SKControl
{
    class Program
    {
        [DllImport("ole32.dll", CallingConvention = CallingConvention.StdCall)]
        public static extern int CoGetClassObject(ref Guid rclsid, uint dwClsContext, IntPtr pServerInfo, ref Guid riid, out IntPtr ppv);

        public bool IsSiteKioskActive(ref bool ab_SKScreensaverActive)
        {

            /*
                returns false, if SiteKiosk is not currently running
                returns true, if SiteKiosk is running and sets 'ab_SKScreensaverActive'
                to true if the screensaver is running and to false if not
            */


            // initialize GUID's for classes and interfaces
            Guid lr_FactoryGuid = typeof(ISiteKioskFactory).GUID;
            Guid lr_FactoryClass = Guid.Parse("1CA0D073-4ABB-4D06-B318-BFFDE38E4903"); 
            Guid lr_SiteKioskGuid = typeof(ISiteKiosk).GUID;

            ISiteKiosk mk_pSiteKiosk;

            // try to get the ISiteKioskFactory interface of the instance
            // of SiteKioskFactoryClass
            IntPtr lk_FactoryPtr = new IntPtr();
            CoGetClassObject(ref lr_FactoryClass, 4, new IntPtr(), ref lr_FactoryGuid, out lk_FactoryPtr);
            if (lk_FactoryPtr == IntPtr.Zero)
                // SiteKiosk is not running
                return false;

            // convert the received IntPtr to the requested ISiteKioskFactory
            // interface
            ISiteKioskFactory lk_Factory = (ISiteKioskFactory)Marshal.GetObjectForIUnknown(lk_FactoryPtr);

            if (lk_Factory == null)
                return false;

            // call CreateSiteKiosk to get the ISiteKiosk interface of the
            // current instance of SiteKiosk
            IntPtr lk_SiteKioskPtr = new IntPtr();
            lk_Factory.CreateSiteKiosk(ref lr_SiteKioskGuid, out lk_SiteKioskPtr);

            if (lk_SiteKioskPtr == IntPtr.Zero)
                return false;

            // convert the received IntPtr to the requested
            // ISiteKioskFactory interface
            mk_pSiteKiosk = (ISiteKiosk)Marshal.GetObjectForIUnknown(lk_SiteKioskPtr);

            if (mk_pSiteKiosk == null)
                return false;

            //check if SK Screensaver is running
            IScreenSaver lr_SKScreensaver = (IScreenSaver)mk_pSiteKiosk.ScreenSaver;
            ab_SKScreensaverActive = lr_SKScreensaver.Active;

            //if SK Screensaver is running logoff to the Windows login screen
            if (ab_SKScreensaverActive)
            {
                mk_pSiteKiosk.LogoffWindows();
            }

            return true;
        }
 
        static int Main(string[] args)
        {
            bool lb_ReturnValue = false;
            bool lb_SKScreensaverActive = false;

            Program lk_Prog = new Program();
            lb_ReturnValue = lk_Prog.IsSiteKioskActive(ref lb_SKScreensaverActive);

            //returns 1 if SiteKiosk is not running
            if (!lb_ReturnValue)
                return 1;
            //returns 2 if SiteKiosk is running and screensaver is not active
            if (lb_ReturnValue && !lb_SKScreensaverActive)
                return 2;
            //returns 3 if SiteKiosk is running and screensaver is active
            if (lb_ReturnValue && lb_SKScreensaverActive)
                return 3;

            //default return value, none of the above applied
            return 0;
        }
    }
}
